/// <reference types="express" />
import * as express from 'express';
import { Client } from './models/client';
import { Token } from './models/token';
export { Client } from './models/client';
export { Token } from './models/token';
export { OAuth2FrameworkRouter } from './router';
export declare class OAuth2Framework {
    model: {
        findClient: (client_id: string, request: express.Request) => Promise<Client>;
        register: (client_id: string, emailAddress: string, username: string, password: string, request: express.Request) => Promise<boolean>;
        resetPassword: (client_id: string, username: string, password: string, request: express.Request) => Promise<boolean>;
        sendForgotPasswordEmail: (client_id: string, username: string, resetPasswordUrl: string, request: express.Request) => Promise<boolean>;
        sendVerificationEmail: (client_id: string, emailAddress: string, username: string, verificationUrl: string, request: express.Request) => Promise<boolean>;
        validateCredentials: (client_id: string, username: string, password: string, request: express.Request) => Promise<boolean>;
        verify: (client_id: string, username: string, request: express.Request) => Promise<boolean>;
        generateCode(client_id: string, username: string, scopes: string[], request: express.Request): Promise<string>;
        validateCode(code: string, request: express.Request): Promise<Token>;
        generateAccessToken(client_id: string, username: string, scopes: string[], request: express.Request): Promise<string>;
        validateAccessToken(code: string, request: express.Request): Promise<Token>;
    };
    secret: string;
    constructor(model: {
        findClient: (client_id: string, request: express.Request) => Promise<Client>;
        register: (client_id: string, emailAddress: string, username: string, password: string, request: express.Request) => Promise<boolean>;
        resetPassword: (client_id: string, username: string, password: string, request: express.Request) => Promise<boolean>;
        sendForgotPasswordEmail: (client_id: string, username: string, resetPasswordUrl: string, request: express.Request) => Promise<boolean>;
        sendVerificationEmail: (client_id: string, emailAddress: string, username: string, verificationUrl: string, request: express.Request) => Promise<boolean>;
        validateCredentials: (client_id: string, username: string, password: string, request: express.Request) => Promise<boolean>;
        verify: (client_id: string, username: string, request: express.Request) => Promise<boolean>;
        generateCode(client_id: string, username: string, scopes: string[], request: express.Request): Promise<string>;
        validateCode(code: string, request: express.Request): Promise<Token>;
        generateAccessToken(client_id: string, username: string, scopes: string[], request: express.Request): Promise<string>;
        validateAccessToken(code: string, request: express.Request): Promise<Token>;
    }, secret: string);
    authorizationRequest(response_type: string, client_id: string, redirect_uri: string, scopes: string[], state: string, username: string, password: string, request: express.Request): Promise<string>;
    accessTokenRequest(grant_type: string, code: string, redirect_uri: string, client_id: string, client_secret: string, username: string, password: string, scopes: string[], request: express.Request): Promise<string>;
    validateAccessToken(access_token: string, request: express.Request): Promise<boolean>;
    decodeAccessToken(access_token: string, request: express.Request): Promise<Token>;
    forgotPasswordRequest(client_id: string, username: string, response_type: string, redirect_uri: string, state: string, request: express.Request): Promise<boolean>;
    emailVerificationRequest(token: string, request: express.Request): Promise<boolean>;
    registerRequest(client_id: string, emailAddress: string, username: string, password: string, response_type: string, redirect_uri: string, state: string, request: express.Request): Promise<boolean>;
    resetPasswordRequest(token: string, password: string, request: express.Request): Promise<boolean>;
    decodeResetPasswordToken(token: string): Promise<any>;
    decodeEmailVerificationToken(token: string): Promise<any>;
    private generateResetPasswordToken(client_id, username, return_url);
    private generateEmailVerificationToken(client_id, username, return_url);
    private decodeJWT(jwt);
}
