"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const chai_1 = require("chai");
require("mocha");
const index_1 = require("./index");
const client_1 = require("./models/client");
const token_1 = require("./models/token");
describe('Tests', () => {
    let framework = null;
    describe('authorizationRequest', () => {
        it('should throw error given invalid response_type', () => __awaiter(this, void 0, void 0, function* () {
            framework = new index_1.OAuth2Framework({
                findClient: (client_id) => {
                    return Promise.resolve(new client_1.Client(null, null, null, null, null, null, null));
                },
                register: null,
                resetPassword: null,
                sendForgotPasswordEmail: null,
                sendVerificationEmail: null,
                validateCredentials: null,
                verify: null,
                generateCode: null,
                validateCode: null,
                generateAccessToken: null,
                validateAccessToken: null,
            }, null);
            try {
                yield framework.authorizationRequest('invalid response_type', 'client_id', 'redirect_uri', ['scope'], 'state', 'username', 'password', null);
                throw new Error('Expected Error');
            }
            catch (err) {
                chai_1.expect(err.message).to.be.equal('Invalid response_type');
            }
        }));
        it('should return null given invalid credentials', () => __awaiter(this, void 0, void 0, function* () {
            framework = new index_1.OAuth2Framework({
                findClient: (client_id) => {
                    return Promise.resolve(new client_1.Client(null, null, null, ['scope'], ['redirect_uri'], null, null));
                },
                register: null,
                resetPassword: null,
                sendForgotPasswordEmail: null,
                sendVerificationEmail: null,
                validateCredentials: (client_id, username, password) => {
                    return Promise.resolve(false);
                },
                verify: null,
                generateCode: null,
                validateCode: null,
                generateAccessToken: null,
                validateAccessToken: null,
            }, null);
            const code = yield framework.authorizationRequest('code', 'client_id', 'redirect_uri', ['scope'], 'state', 'username1', 'password1', null);
            chai_1.expect(code).to.be.null;
        }));
        it('should throw error given invalid client_id', () => __awaiter(this, void 0, void 0, function* () {
            framework = new index_1.OAuth2Framework({
                findClient: (client_id) => {
                    return Promise.resolve(null);
                },
                register: null,
                resetPassword: null,
                sendForgotPasswordEmail: null,
                sendVerificationEmail: null,
                validateCredentials: null,
                verify: null,
                generateCode: null,
                validateCode: null,
                generateAccessToken: null,
                validateAccessToken: null,
            }, null);
            try {
                yield framework.authorizationRequest('code', 'invalid client_id', 'redirect_uri', ['scope'], 'state', 'username', 'password', null);
                throw new Error('Expected Error');
            }
            catch (err) {
                chai_1.expect(err.message).to.be.equal('Invalid client_id');
            }
        }));
        it('should throw error given invalid redirect_uri', () => __awaiter(this, void 0, void 0, function* () {
            framework = new index_1.OAuth2Framework({
                findClient: (client_id) => {
                    return Promise.resolve(new client_1.Client(null, null, null, null, ['redirect_uri'], null, null));
                },
                register: null,
                resetPassword: null,
                sendForgotPasswordEmail: null,
                sendVerificationEmail: null,
                validateCredentials: null,
                verify: null,
                generateCode: null,
                validateCode: null,
                generateAccessToken: null,
                validateAccessToken: null,
            }, null);
            try {
                yield framework.authorizationRequest('code', 'client_id', 'invalid redirect_uri', ['scope'], 'state', 'username', 'password', null);
                throw new Error('Expected Error');
            }
            catch (err) {
                chai_1.expect(err.message).to.be.equal('Invalid redirect_uri');
            }
        }));
        it('should throw error given invalid scopes', () => __awaiter(this, void 0, void 0, function* () {
            framework = new index_1.OAuth2Framework({
                findClient: (client_id) => {
                    return Promise.resolve(new client_1.Client(null, null, null, ['scope'], ['redirect_uri'], null, null));
                },
                register: null,
                resetPassword: null,
                sendForgotPasswordEmail: null,
                sendVerificationEmail: null,
                validateCredentials: null,
                verify: null,
                generateCode: null,
                validateCode: null,
                generateAccessToken: null,
                validateAccessToken: null,
            }, null);
            try {
                yield framework.authorizationRequest('code', 'client_id', 'redirect_uri', ['invalid-scope'], 'state', 'username', 'password', null);
                throw new Error('Expected Error');
            }
            catch (err) {
                chai_1.expect(err.message).to.be.equal('Invalid scopes');
            }
        }));
        it('Authorization Code Grant: should return code', () => __awaiter(this, void 0, void 0, function* () {
            framework = new index_1.OAuth2Framework({
                findClient: (client_id) => {
                    return Promise.resolve(new client_1.Client(null, null, null, ['scope'], ['redirect_uri'], null, null));
                },
                register: null,
                resetPassword: null,
                sendForgotPasswordEmail: null,
                sendVerificationEmail: null,
                validateCredentials: (client_id, username, password) => {
                    return Promise.resolve(true);
                },
                verify: null,
                generateCode: (client_id, username, scopes) => {
                    return Promise.resolve(`${client_id}|${username}|${scopes.join(',')}`);
                },
                validateCode: null,
                generateAccessToken: null,
                validateAccessToken: null,
            }, 'secret');
            const code = yield framework.authorizationRequest('code', 'client_id', 'redirect_uri', ['scope'], 'state', 'username', 'password', null);
            chai_1.expect(code).to.be.not.null;
        }));
        it('Implicit Grant: should return access_token', () => __awaiter(this, void 0, void 0, function* () {
            framework = new index_1.OAuth2Framework({
                findClient: (client_id) => {
                    return Promise.resolve(new client_1.Client(null, null, null, ['scope'], ['redirect_uri'], null, null));
                },
                register: null,
                resetPassword: null,
                sendForgotPasswordEmail: null,
                sendVerificationEmail: null,
                validateCredentials: (client_id, username, password) => {
                    return Promise.resolve(true);
                },
                verify: null,
                generateCode: null,
                validateCode: null,
                generateAccessToken: (client_id, username, scopes) => {
                    return Promise.resolve(`${client_id}|${username}|${scopes.join(',')}`);
                },
                validateAccessToken: null,
            }, 'secret');
            const accessToken = yield framework.authorizationRequest('token', 'client_id', 'redirect_uri', ['scope'], 'state', 'username', 'password', null);
            chai_1.expect(accessToken).to.be.not.null;
        }));
    });
    describe('accessTokenRequest', () => {
        it('should throw error given invalid grant_type', () => __awaiter(this, void 0, void 0, function* () {
            framework = new index_1.OAuth2Framework({
                findClient: (client_id) => {
                    return Promise.resolve(new client_1.Client(null, null, null, null, null, null, null));
                },
                register: null,
                resetPassword: null,
                sendForgotPasswordEmail: null,
                sendVerificationEmail: null,
                validateCredentials: null,
                verify: null,
                generateCode: null,
                validateCode: null,
                generateAccessToken: null,
                validateAccessToken: null,
            }, null);
            try {
                yield framework.accessTokenRequest('invalid grant_type', 'code', 'redirect_uri', 'client_id', 'client_secret', null, null, null, null);
                throw new Error('Expected Error');
            }
            catch (err) {
                chai_1.expect(err.message).to.be.equal('Invalid grant_type');
            }
        }));
        it('should throw error given invalid client_id', () => __awaiter(this, void 0, void 0, function* () {
            framework = new index_1.OAuth2Framework({
                findClient: (client_id) => {
                    return Promise.resolve(null);
                },
                register: null,
                resetPassword: null,
                sendForgotPasswordEmail: null,
                sendVerificationEmail: null,
                validateCredentials: null,
                verify: null,
                generateCode: null,
                validateCode: null,
                generateAccessToken: null,
                validateAccessToken: null,
            }, null);
            try {
                yield framework.accessTokenRequest('authorization_code', 'code', 'redirect_uri', 'invalid client_id', 'client_secret', null, null, null, null);
                throw new Error('Expected Error');
            }
            catch (err) {
                chai_1.expect(err.message).to.be.equal('Invalid client_id');
            }
        }));
        it('should throw error given invalid redirect_uri', () => __awaiter(this, void 0, void 0, function* () {
            framework = new index_1.OAuth2Framework({
                findClient: (client_id) => {
                    return Promise.resolve(new client_1.Client(null, null, null, null, ['redirect_uri'], null, null));
                },
                register: null,
                resetPassword: null,
                sendForgotPasswordEmail: null,
                sendVerificationEmail: null,
                validateCredentials: null,
                verify: null,
                generateCode: null,
                validateCode: null,
                generateAccessToken: null,
                validateAccessToken: null,
            }, null);
            try {
                yield framework.accessTokenRequest('authorization_code', 'code', 'invalid redirect_uri', 'client_id', 'client_secret', null, null, null, null);
                throw new Error('Expected Error');
            }
            catch (err) {
                chai_1.expect(err.message).to.be.equal('Invalid redirect_uri');
            }
        }));
        it('Authorization Code Grant: should throw error given invalid code', () => __awaiter(this, void 0, void 0, function* () {
            framework = new index_1.OAuth2Framework({
                findClient: (client_id) => {
                    return Promise.resolve(new client_1.Client(null, null, null, null, ['redirect_uri'], null, null));
                },
                register: null,
                resetPassword: null,
                sendForgotPasswordEmail: null,
                sendVerificationEmail: null,
                validateCredentials: null,
                verify: null,
                generateCode: null,
                validateCode: (code) => {
                    return Promise.resolve(null);
                },
                generateAccessToken: null,
                validateAccessToken: null,
            }, null);
            try {
                yield framework.accessTokenRequest('authorization_code', 'invalid code', 'redirect_uri', 'client_id', 'client_secret', null, null, null, null);
                throw new Error('Expected Error');
            }
            catch (err) {
                chai_1.expect(err.message).to.be.equal('Invalid code');
            }
        }));
        it('Authorization Code Grant: should throw error given valid access token instead of valid code', () => __awaiter(this, void 0, void 0, function* () {
            framework = new index_1.OAuth2Framework({
                findClient: (client_id) => {
                    return Promise.resolve(new client_1.Client(null, null, null, null, ['redirect_uri'], null, null));
                },
                register: null,
                resetPassword: null,
                sendForgotPasswordEmail: null,
                sendVerificationEmail: null,
                validateCredentials: (client_id, username, password) => {
                    return Promise.resolve(true);
                },
                verify: null,
                generateCode: null,
                validateCode: (code) => {
                    return Promise.resolve(null);
                },
                generateAccessToken: (client_id, username, scopes) => {
                    return Promise.resolve(`${client_id}|${username}|${scopes.join(',')}`);
                },
                validateAccessToken: null,
            }, 'secret');
            try {
                const accessToken = yield framework.accessTokenRequest('password', 'code1', 'redirect_uri', 'client_id', 'client_secret', 'username', 'password', [], null);
                yield framework.accessTokenRequest('authorization_code', accessToken, 'redirect_uri', 'client_id', 'client_secret', null, null, null, null);
                throw new Error('Expected Error');
            }
            catch (err) {
                chai_1.expect(err.message).to.be.equal('Invalid code');
            }
        }));
        it('Authorization Code Grant: should throw error given invalid client_secret', () => __awaiter(this, void 0, void 0, function* () {
            framework = new index_1.OAuth2Framework({
                findClient: (client_id) => {
                    return Promise.resolve(new client_1.Client(null, null, null, ['scope'], ['redirect_uri'], null, null));
                },
                register: null,
                resetPassword: null,
                sendForgotPasswordEmail: null,
                sendVerificationEmail: null,
                validateCredentials: (client_id, username, password) => {
                    return Promise.resolve(true);
                },
                verify: null,
                generateCode: (client_id, username, scopes) => {
                    return Promise.resolve(`${client_id}|${username}|${scopes.join(',')}`);
                },
                validateCode: (code) => {
                    return Promise.resolve(new token_1.Token(null, null, null));
                },
                generateAccessToken: null,
                validateAccessToken: null,
            }, 'secret');
            try {
                const code = yield framework.authorizationRequest('code', 'client_id', 'redirect_uri', ['scope'], 'state', 'username', 'password', null);
                yield framework.accessTokenRequest('authorization_code', code, 'redirect_uri', 'client_id', 'invalid client_secret', null, null, null, null);
                throw new Error('Expected Error');
            }
            catch (err) {
                chai_1.expect(err.message).to.be.equal('Invalid client_secret');
            }
        }));
        it('Authorization Code Grant: should return access token', () => __awaiter(this, void 0, void 0, function* () {
            framework = new index_1.OAuth2Framework({
                findClient: (client_id) => {
                    return Promise.resolve(new client_1.Client(null, null, 'client_secret', ['scope'], ['redirect_uri'], null, null));
                },
                register: null,
                resetPassword: null,
                sendForgotPasswordEmail: null,
                sendVerificationEmail: null,
                validateCredentials: (client_id, username, password) => {
                    return Promise.resolve(true);
                },
                verify: null,
                generateCode: (client_id, username, scopes) => {
                    return Promise.resolve(`${client_id}|${username}|${scopes.join(',')}`);
                },
                validateCode: (code) => {
                    return Promise.resolve(new token_1.Token(null, null, []));
                },
                generateAccessToken: (client_id, username, scopes) => {
                    return Promise.resolve(`${client_id}|${username}|${scopes.join(',')}`);
                },
                validateAccessToken: null,
            }, 'secret');
            const code1 = yield framework.authorizationRequest('code', 'client_id', 'redirect_uri', ['scope'], 'state', 'username', 'password', null);
            const accessToken = yield framework.accessTokenRequest('authorization_code', code1, 'redirect_uri', 'client_id', 'client_secret', null, null, null, null);
            chai_1.expect(accessToken).to.be.not.null;
        }));
        it('Resource Owner Password Credentials Grant: should return null given invalid credentials', () => __awaiter(this, void 0, void 0, function* () {
            framework = new index_1.OAuth2Framework({
                findClient: (client_id) => {
                    return Promise.resolve(new client_1.Client(null, null, null, null, ['redirect_uri'], null, null));
                },
                register: null,
                resetPassword: null,
                sendForgotPasswordEmail: null,
                sendVerificationEmail: null,
                validateCredentials: (client_id, username, password) => {
                    return Promise.resolve(false);
                },
                verify: null,
                generateCode: null,
                validateCode: null,
                generateAccessToken: null,
                validateAccessToken: null,
            }, null);
            const accessToken = yield framework.accessTokenRequest('password', 'code1', 'redirect_uri', 'client_id', 'client_secret', 'username', 'password', [], null);
            chai_1.expect(accessToken).to.be.null;
        }));
        it('Resource Owner Password Credentials Grant: should return access token given valid credentials', () => __awaiter(this, void 0, void 0, function* () {
            framework = new index_1.OAuth2Framework({
                findClient: (client_id) => {
                    return Promise.resolve(new client_1.Client(null, null, null, ['scope'], ['redirect_uri'], null, null));
                },
                register: null,
                resetPassword: null,
                sendForgotPasswordEmail: null,
                sendVerificationEmail: null,
                validateCredentials: (client_id, username, password) => {
                    return Promise.resolve(true);
                },
                verify: null,
                generateCode: null,
                validateCode: null,
                generateAccessToken: (client_id, username, scopes) => {
                    return Promise.resolve(`${client_id}|${username}|${scopes.join(',')}`);
                },
                validateAccessToken: null,
            }, 'secret');
            const accessToken = yield framework.accessTokenRequest('password', 'code1', 'redirect_uri', 'client_id', 'client_secret', 'username', 'password', ['scope'], null);
            chai_1.expect(accessToken).to.be.not.null;
        }));
    });
    describe('validateAccessToken', () => {
        it('should return false given invalid token', () => __awaiter(this, void 0, void 0, function* () {
            framework = new index_1.OAuth2Framework({
                findClient: null,
                register: null,
                resetPassword: null,
                sendForgotPasswordEmail: null,
                sendVerificationEmail: null,
                validateCredentials: null,
                verify: null,
                generateCode: null,
                validateCode: null,
                generateAccessToken: (client_id, username, scopes) => {
                    return Promise.resolve(`${client_id}|${username}|${scopes.join(',')}`);
                },
                validateAccessToken: (access_token) => {
                    return Promise.resolve(null);
                },
            }, null);
            const result = yield framework.validateAccessToken('invalid token', null);
            chai_1.expect(result).to.be.false;
        }));
        it('should return false given code', function* () {
            framework = new index_1.OAuth2Framework({
                findClient: (client_id) => {
                    return Promise.resolve(new client_1.Client(null, null, 'client_secret', null, ['redirect_uri'], null, null));
                },
                register: null,
                resetPassword: null,
                sendForgotPasswordEmail: null,
                sendVerificationEmail: null,
                validateCredentials: (client_id, username, password) => {
                    return Promise.resolve(true);
                },
                verify: null,
                generateCode: null,
                validateCode: null,
                generateAccessToken: null,
                validateAccessToken: null,
            }, null);
            const code = yield framework.authorizationRequest('code', 'client_id', 'redirect_uri', ['scope'], 'state', 'username', 'password', null);
            const result = yield framework.validateAccessToken(code, null);
            chai_1.expect(result).to.be.false;
        });
        it('should return true given valid token', () => __awaiter(this, void 0, void 0, function* () {
            framework = new index_1.OAuth2Framework({
                findClient: (client_id) => {
                    return Promise.resolve(new client_1.Client(null, null, null, null, ['redirect_uri'], null, null));
                },
                register: null,
                resetPassword: null,
                sendForgotPasswordEmail: null,
                sendVerificationEmail: null,
                validateCredentials: (client_id, username, password) => {
                    return Promise.resolve(true);
                },
                verify: null,
                generateCode: null,
                validateCode: null,
                generateAccessToken: (client_id, username, scopes) => {
                    return Promise.resolve(`${client_id}|${username}|${scopes.join(',')}`);
                },
                validateAccessToken: (access_token) => {
                    return Promise.resolve(new token_1.Token(null, null, null));
                },
            }, 'secret');
            const accessToken = yield framework.accessTokenRequest('password', 'code1', 'redirect_uri', 'client_id', 'client_secret', 'username', 'password', [], null);
            const result = yield framework.validateAccessToken(accessToken, null);
            chai_1.expect(result).to.be.true;
        }));
    });
    describe('forgotPasswordRequest', () => {
        it('should throw error given invalid client_id', () => __awaiter(this, void 0, void 0, function* () {
            framework = new index_1.OAuth2Framework({
                findClient: (client_id) => {
                    return Promise.resolve(null);
                },
                register: null,
                resetPassword: null,
                sendForgotPasswordEmail: null,
                sendVerificationEmail: null,
                validateCredentials: null,
                verify: null,
                generateCode: null,
                validateCode: null,
                generateAccessToken: null,
                validateAccessToken: null,
            }, null);
            try {
                yield framework.forgotPasswordRequest('invalid client_id', 'username', 'response_type', 'redirect_uri', '', null);
                throw new Error('Expected Error');
            }
            catch (err) {
                chai_1.expect(err.message).to.be.equal('Invalid client_id');
            }
        }));
        it('should throw error given client disabled forgot password', () => __awaiter(this, void 0, void 0, function* () {
            framework = new index_1.OAuth2Framework({
                findClient: (client_id) => {
                    return Promise.resolve(new client_1.Client(null, null, null, null, null, false, null));
                },
                register: null,
                resetPassword: null,
                sendForgotPasswordEmail: null,
                sendVerificationEmail: null,
                validateCredentials: null,
                verify: null,
                generateCode: null,
                validateCode: null,
                generateAccessToken: null,
                validateAccessToken: null,
            }, null);
            try {
                yield framework.forgotPasswordRequest('client_id', 'username1', 'response_type', 'redirect_uri', '', null);
                throw new Error('Expected Error');
            }
            catch (err) {
                chai_1.expect(err.message).to.be.equal('Function not enabled for client');
            }
        }));
        it('should return false given sendForgotPasswordEmail fails', () => __awaiter(this, void 0, void 0, function* () {
            framework = new index_1.OAuth2Framework({
                findClient: (client_id) => {
                    return Promise.resolve(new client_1.Client(null, null, null, null, null, true, null));
                },
                register: null,
                resetPassword: null,
                sendForgotPasswordEmail: (client_id, username, resetPasswordUrl) => {
                    return Promise.resolve(false);
                },
                sendVerificationEmail: null,
                validateCredentials: null,
                verify: null,
                generateCode: null,
                validateCode: null,
                generateAccessToken: null,
                validateAccessToken: null,
            }, 'secret');
            const result = yield framework.forgotPasswordRequest('client_id1', 'username1', 'response_type', 'redirect_uri', '', null);
            chai_1.expect(result).to.be.false;
        }));
        it('should return true given sendForgotPasswordEmail succeeds', () => __awaiter(this, void 0, void 0, function* () {
            framework = new index_1.OAuth2Framework({
                findClient: (client_id) => {
                    return Promise.resolve(new client_1.Client(null, null, null, null, null, true, null));
                },
                register: null,
                resetPassword: null,
                sendForgotPasswordEmail: (client_id, username, resetPasswordUrl) => {
                    return Promise.resolve(true);
                },
                sendVerificationEmail: null,
                validateCredentials: null,
                verify: null,
                generateCode: null,
                validateCode: null,
                generateAccessToken: null,
                validateAccessToken: null,
            }, 'secret');
            const result = yield framework.forgotPasswordRequest('client_id', 'username1', 'response_type', 'redirect_uri', '', null);
            chai_1.expect(result).to.be.true;
        }));
    });
});
//# sourceMappingURL=index.test.js.map