"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const express = require("express");
const fs = require("fs");
const Handlebars = require("handlebars");
const path = require("path");
const index_1 = require("./index");
function OAuth2FrameworkRouter(model, loginPagePath, forgotPasswordPagePath, forgotPasswordSuccessPagePath, forgotPasswordFailurePagePath, resetPasswordPagePath, registerPagePath, registerSuccessPagePath, registerFailurePagePath, emailVerficationSuccessPagePath, emailVerficationFailurePagePath, secret) {
    const router = express.Router();
    const framework = new index_1.OAuth2Framework(model, secret);
    router.get('/authorize', (req, res) => __awaiter(this, void 0, void 0, function* () {
        try {
            const client = yield framework.model.findClient(req.query.client_id, req);
            if (!client) {
                throw new Error('Invalid client_id');
            }
            renderPage(res, loginPagePath || path.join(__dirname, 'views/login.handlebars'), {
                client,
                query: req.query,
            }, 200);
        }
        catch (err) {
            res.status(500).send(err.message);
        }
    }));
    router.post('/authorize', (req, res) => __awaiter(this, void 0, void 0, function* () {
        try {
            const result = yield framework.authorizationRequest(req.query.response_type, req.query.client_id, req.query.redirect_uri, req.query.scope ? [req.query.scope] : [], req.query.state, req.body.username, req.body.password, req);
            const client = yield framework.model.findClient(req.query.client_id, req);
            if (!result) {
                renderPage(res, loginPagePath || path.join(__dirname, 'views/login.handlebars'), {
                    client,
                    message: 'Invalid login credentials',
                    query: req.query,
                }, 200);
                return;
            }
            if (req.query.response_type === 'code') {
                res.redirect(`${req.query.redirect_uri}?code=${result}&state=${req.query.state}`);
            }
            else if (req.query.response_type === 'token') {
                res.redirect(`${req.query.redirect_uri}?access_token=${result}&state=${req.query.state}`);
            }
        }
        catch (err) {
            res.status(500).send(err.message);
        }
    }));
    router.post('/token', (req, res) => __awaiter(this, void 0, void 0, function* () {
        try {
            const accessToken = yield framework.accessTokenRequest(req.body.grant_type, req.body.code, req.body.redirect_uri, req.body.client_id, req.body.client_secret, req.body.username, req.body.password, [req.body.scope], req);
            res.json({
                access_token: accessToken,
            });
        }
        catch (err) {
            res.status(500).send(err.message);
        }
    }));
    router.post('/validate', (req, res) => __awaiter(this, void 0, void 0, function* () {
        try {
            const authorizationHeader = req.get('Authorization');
            if (!authorizationHeader || authorizationHeader.split(' ')[0].toLowerCase() !== 'bearer') {
                throw new Error('Invalid header');
            }
            const access_token = authorizationHeader.split(' ')[1];
            const valid = yield framework.validateAccessToken(access_token, null);
            res.json({
                valid,
            });
        }
        catch (err) {
            res.status(500).send(err.message);
        }
    }));
    router.get('/user', (req, res) => __awaiter(this, void 0, void 0, function* () {
        try {
            const authorizationHeader = req.get('Authorization');
            if (!authorizationHeader || authorizationHeader.split(' ')[0].toLowerCase() !== 'bearer') {
                throw new Error('Invalid header');
            }
            const access_token = authorizationHeader.split(' ')[1];
            const valid = yield framework.validateAccessToken(access_token, null);
            if (valid) {
                const decodedToken = yield framework.decodeAccessToken(access_token, null);
                res.json(decodedToken);
            }
            else {
                res.json(null);
            }
        }
        catch (err) {
            res.status(500).send(err.message);
        }
    }));
    router.get('/forgot-password', (req, res) => __awaiter(this, void 0, void 0, function* () {
        try {
            const client = yield framework.model.findClient(req.query.client_id, req);
            if (!client) {
                throw new Error('Invalid client_id');
            }
            renderPage(res, forgotPasswordPagePath || path.join(__dirname, 'views/forgot-password.handlebars'), {
                client,
                query: req.query,
            }, 200);
        }
        catch (err) {
            res.status(500).send(err.message);
        }
    }));
    router.post('/forgot-password', (req, res) => __awaiter(this, void 0, void 0, function* () {
        try {
            const client = yield framework.model.findClient(req.query.client_id, req);
            if (!client) {
                throw new Error('Invalid client_id');
            }
            try {
                const result = yield framework.forgotPasswordRequest(req.query.client_id, req.body.username, req.query.response_type, req.query.redirect_uri, req.query.state, req);
                if (result) {
                    renderPage(res, forgotPasswordSuccessPagePath || path.join(__dirname, 'views/forgot-password-success.handlebars'), {
                        client,
                        query: req.query,
                    }, 200);
                }
                else {
                    renderPage(res, forgotPasswordFailurePagePath || path.join(__dirname, 'views/forgot-password-failure.handlebars'), {
                        client,
                        query: req.query,
                    }, 200);
                }
            }
            catch (err) {
                renderPage(res, forgotPasswordPagePath || path.join(__dirname, 'views/forgot-password.handlebars'), {
                    client,
                    message: err.message,
                    query: req.query,
                }, 200);
            }
        }
        catch (err) {
            res.status(500).send(err.message);
        }
    }));
    router.get('/reset-password', (req, res) => __awaiter(this, void 0, void 0, function* () {
        try {
            const decodedToken = yield framework.decodeResetPasswordToken(req.query.token);
            if (!decodedToken) {
                throw new Error('Invalid token');
            }
            const client = yield framework.model.findClient(decodedToken.client_id, req);
            if (!client) {
                throw new Error('Invalid client_id');
            }
            renderPage(res, resetPasswordPagePath || path.join(__dirname, 'views/reset-password.handlebars'), {
                client,
                query: req.query,
            }, 200);
        }
        catch (err) {
            res.status(500).send(err.message);
        }
    }));
    router.post('/reset-password', (req, res) => __awaiter(this, void 0, void 0, function* () {
        try {
            const decodedToken = yield framework.decodeResetPasswordToken(req.query.token);
            if (!decodedToken) {
                throw new Error('Invalid token');
            }
            const client = yield framework.model.findClient(decodedToken.client_id, req);
            if (!client) {
                throw new Error('Invalid client_id');
            }
            try {
                const result = yield framework.resetPasswordRequest(req.query.token, req.body.password, req);
                if (result) {
                    res.redirect(decodedToken.return_url);
                }
                else {
                    renderPage(res, resetPasswordPagePath || path.join(__dirname, 'views/reset-password.handlebars'), {
                        client,
                        query: req.query,
                    }, 200);
                }
            }
            catch (err) {
                renderPage(res, resetPasswordPagePath || path.join(__dirname, 'views/reset-password.handlebars'), {
                    client,
                    message: err.message,
                    query: req.query,
                }, 200);
            }
        }
        catch (err) {
            res.status(500).send(err.message);
        }
    }));
    router.get('/register', (req, res) => __awaiter(this, void 0, void 0, function* () {
        try {
            const client = yield framework.model.findClient(req.query.client_id, req);
            if (!client) {
                throw new Error('Invalid client_id');
            }
            renderPage(res, registerPagePath || path.join(__dirname, 'views/register.handlebars'), {
                client,
                query: req.query,
            }, 200);
        }
        catch (err) {
            res.status(500).send(err.message);
        }
    }));
    router.post('/register', (req, res) => __awaiter(this, void 0, void 0, function* () {
        try {
            const client = yield framework.model.findClient(req.query.client_id, req);
            if (!client) {
                throw new Error('Invalid client_id');
            }
            try {
                const result = yield framework.registerRequest(req.query.client_id, req.body.emailAddress, req.body.username, req.body.password, req.query.response_type, req.query.redirect_uri, req.query.state, req);
                if (result) {
                    renderPage(res, registerSuccessPagePath || path.join(__dirname, 'views/register-success.handlebars'), {
                        client,
                        query: req.query,
                    }, 200);
                }
                else {
                    renderPage(res, registerFailurePagePath || path.join(__dirname, 'views/register-failure.handlebars'), {
                        client,
                        query: req.query,
                    }, 200);
                }
            }
            catch (err) {
                renderPage(res, registerPagePath || path.join(__dirname, 'views/register.handlebars'), {
                    client,
                    message: err.message,
                    query: req.query,
                }, 200);
            }
        }
        catch (err) {
            res.status(500).send(err.message);
        }
    }));
    router.get('/email-verification', (req, res) => __awaiter(this, void 0, void 0, function* () {
        try {
            const decodedToken = yield framework.decodeEmailVerificationToken(req.query.token);
            if (!decodedToken) {
                throw new Error('Invalid token');
            }
            const client = yield framework.model.findClient(decodedToken.client_id, req);
            if (!client) {
                throw new Error('Invalid client_id');
            }
            const result = yield framework.emailVerificationRequest(req.query.token, req);
            req.query.return_url = decodedToken.return_url;
            if (result) {
                renderPage(res, emailVerficationSuccessPagePath || path.join(__dirname, 'views/email-verification-success.handlebars'), {
                    client,
                    query: req.query,
                }, 200);
            }
            else {
                renderPage(res, emailVerficationFailurePagePath || path.join(__dirname, 'views/email-verification-failure.handlebars'), {
                    client,
                    query: req.query,
                }, 200);
            }
        }
        catch (err) {
            res.status(500).send(err.message);
        }
    }));
    return router;
}
exports.OAuth2FrameworkRouter = OAuth2FrameworkRouter;
function renderPage(res, htmlFile, data, status) {
    fs.readFile(htmlFile, 'utf8', (err, html) => {
        if (err) {
            res.status(400).send(err.message);
            return;
        }
        const template = Handlebars.compile(html);
        const result = template(data);
        res.status(status).send(result);
    });
}
//# sourceMappingURL=router.js.map